# ~*~ coding: utf-8
from utils import translate 

class InvalidValue(Exception):
    pass

class field(object):

    def getLegacy(klass):
        try:
            return klass.legacy
        except AttributeError:
            return None
    getLegacy = classmethod(getLegacy)

    def getName(klass):
        return klass.__name__
    getName = classmethod(getName)

    def getTitleI18N(klass, lang):
        return translate(None, "field_"+klass.getName(), lang)
    getTitleI18N = classmethod(getTitleI18N)

    def getKeys(klass):
        return klass.choices.keys()
    getKeys = classmethod(getKeys)

    def view(klass, obj=None, trLang='en'):
        r = u'<div class="field view">\n'
        tr_title = klass.getTitleI18N(trLang)
        r += '\t<label for="'+klass.__name__+'">' + tr_title + '</label>\n'
        r += '\t<div>'
        fval = klass.default
        if obj is not None:
            fval = klass.getChoiceI18N(klass.get(obj), trLang)
        r += fval
        r += '</div>\n'
        r += '</div>\n'
        return r
    view = classmethod(view)

    def edit(klass, obj=None, trLang='en'):
        r = u'<div class="field edit">\n'
        tr_title = klass.getTitleI18N(trLang)
        r += '\t<label for="'+klass.__name__+'">' + tr_title + '</label>\n'
        r += '\t<div>\n'
        r += klass.select(obj, trLang)
        r += '\t</div>\n'
        r += '</div>\n'
        return r
    edit = classmethod(edit)

    def getItems(klass):
        items = klass.choices.items()
        items.sort(key=lambda k: k[1].get('order'))
        return [ (k, d.get('title')) for k, d in items ]
    getItems = classmethod(getItems)

    def select(klass, obj=None, trLang='en'):
        r = '\t\t<select id="'+klass.__name__+'" name="'+klass.__name__+'">\n'
        fval = klass.default
        if obj is not None:
            fval = klass.get(obj)
        for k, v in klass.getItems():
            r += '\t\t\t<option value="'+k+'"'
            if fval == k:
                r += ' selected="selected"'
            im = k
            if not k: im = v
            r += '>' + klass.getChoiceI18N(k, trLang) + '</option>\n'
        r += '\t\t</select>\n'
        return r
    select = classmethod(select)

    def getChoiceI18N(klass, k, lang):
        if not k: k = klass.choices.get(k).get('title')
        return translate(None, 'fc_'+klass.getName()+'_'+k, lang)
    getChoiceI18N = classmethod(getChoiceI18N)

    def get(klass, obj):
        if hasattr(obj, '_dsstore'):
            return obj._dsstore.get(klass.__name__, klass.default)
        else:
            if obj.has_key(klass.__name__):
                return obj.get(klass.__name__)
    get = classmethod(get)

    def set(klass, obj, val):
        if not klass.choices.has_key(val):
            raise InvalidValue(klass.__name__ + " " + val)
        obj._dsstore.update({klass.__name__: val})
    set = classmethod(set)

class docLanguage(field):
    default = 'et'
    title = u'Document language'
    choices = {
        'et' : {'title': u'Estonian', 'order': 1},
        'ru' : {'title': u'Russian', 'order': 2},
        'fi' : {'title': u'Finnish', 'order': 3},
        'en' : {'title': u'English', 'order': 4}
    }

class residence(field): 
    default = ''
    title = u'Residence'
    legacy = 'elukoht'
    choices = {
        '' : {'title': u'not important', 'order': 1},
        'muu': {'title': u'other', 'order': 2},
        'tallinn': {'title': u'Tallinn/Harjumaa', 'order': 3},
        'idaviru': {'title': u'Ida-Virumaa', 'order': 4}
    }

class socBackground(field): 
    default = ''
    title = u'Social background'
    legacy = 'sots'
    choices = {
        '': {'title': u'not important', 'order': 1},
        'op': {'title': u'Student', 'order': 2},
        'ylop': {'title': u'Undergraduate', 'order': 3},
        'teenist': {'title': u'Teenistuja', 'order': 4},
        'kvaliftooline': {'title': u'qualified worker', 'order': 5},
        'lihttooline': {'title': u'Worker', 'order': 6},
        'tootu': {'title': u'Between jobs', 'order': 7},
        'kodune': {'title': u'Home', 'order': 8},
        'pension': {'title': u'Pensioner', 'order': 9}
    }
class age(field): 
    default = ''
    title = u'Age'
    legacy = 'vanus'
    choices = {
        '': {'title': u'unknown', 'order': 1},
        'kuni18': {'title': u'up to 18', 'order': 2},
        'kuni26': {'title': u'up to 26', 'order': 3},
        'kuni40': {'title': u'up to 40', 'order': 4},
        '41plus': {'title': u'41+', 'order': 5}
    }
class gender(field): 
    default = ''
    title = u'Gender'
    legacy = 'sugu'
    choices = {
        '': {'title': u'unknown', 'order': 1},
        'mees': {'title': u'Male', 'order': 2},
        'naine': {'title': u'Female', 'order': 3}
    }
class motherTongue(field): 
    default = ''
    title = u'Mother tongue'
    legacy = 'emakeel'
    choices = {
        '': {'title': u'unknown', 'order': 1},
        'muud': {'title': u'muud', 'order': 2},
        'vene': {'title': u'vene', 'order': 3},
        'eesti': {'title': u'eesti', 'order': 4},
        'ukraina': {'title': u'ukraina', 'order': 5},
        'valgevene': {'title': u'valgevene', 'order': 6},
        'lati': {'title': u'läti', 'order': 7},
        'leedu': {'title': u'leedu', 'order': 8},
        'saksa': {'title': u'saksa', 'order': 9},
        'soome': {'title': u'soome', 'order': 10},
        'rootsi': {'title': u'rootsi', 'order': 11},
        'inglise': {'title': u'inglise', 'order': 12},
        'jidis': {'title': u'jidiš', 'order': 13},
        'itaalia': {'title': u'itaalia', 'order': 14},
        'jaapani': {'title': u'jaapani', 'order': 15},
        'poola': {'title': u'poola', 'order': 16},
        'hollandi': {'title': u'hollandi', 'order': 17},
        'sloveenia': {'title': u'sloveenia', 'order': 18},
        'hebrea': {'title': u'hebrea', 'order': 19},
        'prantsuse': {'title': u'prantsuse', 'order': 20},
        'katalaani': {'title': u'katalaani', 'order': 21},
        'ungari': {'title': u'ungari', 'order': 22}
    }
class langAtHome(field): 
    default = ''
    title = u'Language used home'
    legacy = 'kodus'
    choices = {
        '': {'title': u'unknown', 'order': 1},
        'muud': {'title': u'muud', 'order': 2},
        'vene': {'title': u'vene', 'order': 3},
        'eesti': {'title': u'eesti', 'order': 4},
        'ukraina': {'title': u'ukraina', 'order': 5},
        'valgevene': {'title': u'valgevene', 'order': 6},
        'lati': {'title': u'läti', 'order': 7},
        'leedu': {'title': u'leedu', 'order': 8},
        'saksa': {'title': u'saksa', 'order': 9},
        'soome': {'title': u'soome', 'order': 10},
        'rootsi': {'title': u'rootsi', 'order': 11},
        'inglise': {'title': u'inglise', 'order': 12},
        'jidis': {'title': u'jidiš', 'order': 13},
        'itaalia': {'title': u'itaalia', 'order': 14},
        'jaapani': {'title': u'jaapani', 'order': 15},
        'poola': {'title': u'poola', 'order': 16},
        'hollandi': {'title': u'hollandi', 'order': 17},
        'sloveenia': {'title': u'sloveenia', 'order': 18},
        'hebrea': {'title': u'hebrea', 'order': 19},
        'prantsuse': {'title': u'prantsuse', 'order': 20},
        'katalaani': {'title': u'katalaani', 'order': 21},
        'ungari': {'title': u'ungari', 'order': 22}
    }
class levOfCommand(field): 
    default = ''
    title = u'Level of command'
    legacy = 'valdamise_tase'
    choices = {
        '': {'title': u'unknown', 'order': 1},
        'A': {'title': u'A', 'order': 2},
        'B': {'title': u'B', 'order': 3},
        'C': {'title': u'C', 'order': 4}
    }
class education(field): 
    default = ''
    title = u'Education'
    legacy = 'haridus'
    choices = {
        '': {'title': u'unknown', 'order': 1},
        'muu': {'title': u'Other', 'order': 2},
        'pohi': {'title': u'Basic', 'order': 3},
        'kesk': {'title': u'Secondary', 'order': 4},
        'korg': {'title': u'Higher', 'order': 5}
    }
class helpingMaterials(field): 
    default = ''
    title = u'Intermediate agent'
    legacy = 'abivahendid'
    choices = {
        '': {'title': u'unknown', 'order': 1},
        'jah': {'title': u'Yes', 'order': 2},
        'ei': {'title': u'No', 'order': 3}
    }

class texttype(field): 
    default = ''
    title = u'Text type'
    legacy = 'texttype'
    choices = {
        '': {'title': u'unknown', 'order': 1},
        'isikiri': {'title': u'Personal letter', 'order': 2},
        'amtkiri': {'title': u'Formal letter', 'order': 3},
        'essee': {'title': u'Essay', 'order': 4},
        'referaat': {'title': u'referaat', 'order': 5},
        'trilumunud': {'title': u'Published in print', 'order': 6},
        'analyys': {'title': u'Analysis', 'order': 7},
        'vastkys': {'title': u'Answer to question', 'order': 8},
        'ymberjutustus': {'title': u'ümberjutustus', 'order': 9},
        'tolge': {'title': u'Translation', 'order': 10},
        'harjutus': {'title': u'Exercise', 'order': 11},
        'muu': {'title': u'Other', 'order': 12}
    }
